/**
 * Created by Renat Fazilyanov(cptrnt@mail.ru) on 23.05.18.
 */
'use strict';

Object.defineProperties( Object, {

    diff: {
        value: function ( oldObj, newObj, includeObject = true ) {

            let diff = {};
            let obj = newObj;

            if( newObj && !oldObj ) {
                diff.type = 'add';
            }

            if( !newObj && oldObj ) {
                diff.type = 'remove';
                obj = oldObj;
            }

            if( newObj && oldObj ) {
                diff.type = 'update';
                diff.diff = oldObj.diffFrom( newObj );
            }


            if( includeObject ){
                diff.data = obj;
            }


            return diff;

        },
        enumerable : false,
        writable : true
    },

    copy: {
        value: function ( obj, safePriveate = false, maxDeep = 5 ) {
            maxDeep--;
            let objCopy = {};
            for(let key of Object.keys(obj)){
                if( !safePriveate || key[0] != '_'){
                    if (Object.prototype.toString.call(obj[key]) === '[object Object]') {
                        objCopy[key] = (maxDeep >= 0 ) ? Object.copy(obj[key], safePriveate, maxDeep) : {};
                    } else {
                        objCopy[key] = obj[key]
                    }
                }
            }
            return objCopy;
        },
        enumerable : false,
        writable : true
    },

    merge: {
        value: function ( sourceObj ) {

            if( arguments.length === 0 ) return {};
            if( arguments.length === 1 ) return arguments[ 0 ];

            let resObject = Object.copy( arguments[ 0 ] );

            for ( let i = 1; i < arguments.length; i++) {
                resObject.mergeFrom( arguments[ i ] );
            }

            return resObject;
        },
        enumerable : false,
        writable : true
    },

    summ: {
        value: function ( sourceObj ) {

            if( arguments.length === 0 ) return {};
            if( arguments.length === 1 ) return arguments[ 0 ];

            let resObject = Object.copy( arguments[ 0 ] );

            for ( let i = 1; i < arguments.length; i++) {
                for( let key in  arguments[ i ]  ){
                    resObject[ key ] = arguments[ i ][ key ];
                }
            }

            return resObject;
        },
        enumerable : false,
        writable : true
    }


});

Object.defineProperties( Object.prototype, {

    pickKeys: {
        value: function ( keys ) {

            if( !Array.isArray( keys) ) return {};

            let obj = {};

            keys.forEach( key => {
                obj[ key ] = this[ key ];
            });

            return obj;
        },
        enumerable : false,
        writable : true
    },

    mergeFrom: {
        value: function ( fromObj, addFromObjKeys = true ) {

            if( !fromObj || typeof fromObj !== 'object' ) return this;

            let keys = Object.keys( this );

            let fromKeys = Object.keys( fromObj );

            // Обходим те кулючи что у нас уже есть!
            keys.forEach( key => {

                fromKeys.delete( key );

                let fromValue = fromObj[ key ];
                if( fromValue === undefined || fromValue === null){
                    return;
                }

                if( typeof this[ key ] !== 'object' ||
                    typeof this[ key ] !== typeof fromValue ||
                    Array.isArray( this[ key ] )){
                    this[ key ] = fromValue;
                    return;
                }

                this[ key ].mergeFrom( fromValue );

            });

            // Добавляем те что есть только у fromObj
            if( addFromObjKeys && fromKeys.length >0  ){

                fromKeys.forEach( key => {
                    this[ key ] = fromObj[ key ];
                });

            }

            return this;

        },
        enumerable : false,
        writable : true
    },

    keysFrom: {
        value: function ( fromObj, exeptKeys  ) {

            if( !fromObj || typeof fromObj !== 'object' ) return this;

            Object.keys( fromObj ).forEach( key => {

                if( !exeptKeys || !Array.isArray( exeptKeys) || exeptKeys.indexOf( key ) === -1 ){
                    this[ key ] = fromObj[ key ];
                }

            });

            return this;

        },
        enumerable : false,
        writable : true
    },

    diffFrom:{
        value: function ( newObj, result = undefined ) {

            let newKeys  = Object.keys( newObj );
            let oldKeys  = Object.keys( this );

            newKeys.forEach( key => {

                oldKeys = oldKeys.delete( key );


                let oldVal = this[ key ];
                let newVal = newObj[ key ];


                if( oldVal === undefined || oldVal === null ){

                    if( !result ){
                        result = {};
                    }

                    result[ key ] = {
                        'to'   : newVal
                    };

                    return;
                }


                let newType = typeof newVal;
                let oldType = typeof oldVal;

                if( newType !== oldType ||
                    newType !== 'object'     &&  oldVal !== newVal ||
                    Array.isArray( newVal ) !==  Array.isArray( oldVal ) ||
                    Array.isArray( newVal )  &&  !oldVal.equal( newVal )  ){

                    if( !result ){
                        result = {};
                    }

                    result[ key ] = {
                        'to'   : newVal,
                        'from' : oldVal
                    };

                    return;

                }


                if( newType === 'object'  && !Array.isArray( newVal ) ){

                    let v = oldVal.diffFrom( newVal );
                    if( v && !result ){
                        result = {};
                    }

                    if( v ){
                        result[ key ] = v;
                    }

                    return;
                }


            });


            if( oldKeys.length > 0 ){
                oldKeys.forEach( key => {

                    let oldVal = this[ key ];

                    if( oldVal !== undefined || oldVal !== null ){

                        if( !result ){
                            result = {};
                        }

                        result[ key ] = {
                            'from'   : oldVal
                        };


                        return;
                    }

                });
            }

            return result;
        },
        enumerable : false,
        writable   : true
    }

} );
